#include <chrono>
#include <iostream>



#include "FuzzySearchDatabase.hpp"
#include "testData.h"


void fillDb(fuzzysearch::Database<>& f)
{
	auto t1 = std::chrono::steady_clock::now();

	int cnt = sizeof(entries) / sizeof(Entry);

	std::vector<std::string> fields(9);

	for (int i = 0; i < cnt; ++i)
	{
		const Entry& e = entries[i];

		fields[0] = e.slug;
		fields[1] = e.name;
		fields[2] = e.brand;
		fields[3] = e.author;

		fields[4] = e.desc;
		fields[5] = e.modslug;
		fields[6] = e.modname;
		fields[7] = e.moddesc;
		fields[8] = e.tags;

		std::string key(e.slug);
		key += ":";
		key += e.name;
		key += ":";
		key += e.brand;
		key += ":";
		key += e.author;
		key += ":";
		key += e.desc;
		key += ":";
		key += e.modslug;
		key += ":";
		key += e.modname;
		key += ":";
		key += e.moddesc;
		key += ":";
		key += e.tags;


		f.addEntry( key, fields );
	}

	auto t2 = std::chrono::steady_clock::now();
	std::chrono::duration<double, std::milli> fp_ms = t2 - t1;

	std::cout << "\nFillDB Time: " << fp_ms.count() << " ms\n";

}

void q(const fuzzysearch::Database<>& f, std::string qs)
{
	std::cout << " --- \n";

	auto t1 = std::chrono::steady_clock::now();
	auto res = f.search( qs );
	auto t2 = std::chrono::steady_clock::now();
	std::chrono::duration<double, std::milli> fp_ms = t2 - t1;

	std::cout << "\nQuery: \"" << qs << "\" Time: " << fp_ms.count() << " ms\n";

	for (auto& a : res) {
		std::cout << "score: " << a.score <<" -> Key: \"" << a.key << "\"\n";
	}
}

int main(int /*argc*/, char** /*argv*/)
{
	fuzzysearch::Database<> f;
	fillDb(f);

	f.setWeights({
		0.8f, // slug
		1.0f, // name
		1.0f, // brand
		1.0f, // author
		0.8f, // desc
		0.8f, // modslug
		1.0f, // modname
		0.8f, // moddesc
		0.8f // tags
		});


	q(f, "vcv vco");
	q(f, "supercell");
	q(f, "super");
	q(f, "grayscale");
	q(f, "scale");
	q(f, "mix");
	q(f, "scale");
	q(f, "huge room");

	q(f, "revrb");
	q(f, "nysti envelope");

	puts( "done" );
}